require 'base64'
require 'json'
require 'logger'
require 'rbconfig'
require 'fileutils'
require 'yaml'

require 'slop'
require 'highline/import'
require 'eventmachine'
require 'em-http-request'
require 'faye/websocket'
require 'clipboard'

require 'forward/core_extensions'

require 'forward/common'
require 'forward/api'
require 'forward/config'
require 'forward/request'
require 'forward/socket'
require 'forward/tunnel'
require 'forward/cli'
require 'forward/version'

module Forward
  class Error < StandardError; end
	# An error occurred with the CLI
	class CLIError < Error; end
	# An error occurred with the Client
	class ClientError < Error; end
	# An error occurred with the Config
	class ConfigError < Error; end
	# An error occurred with the Tunnel
	class TunnelError < Error; end

  SUPPORT_EMAIL = 'support@forwardhq.com'.freeze

  # Helper for determining the host OS
  #
  # Returns simplified and symbolized host os name
  def self.os
    @os ||= begin
      case RbConfig::CONFIG['host_os']
      when /mswin|mingw|cygwin/
        :windows
      when /darwin/
        :osx
      when /linux|bsd/
        :unix
      else
        :unknown
      end
    end
  end

  # Helper to determine if host OS is windows
  #
  # Returns Boolean
  def self.windows?
    @windows ||= os == :windows
  end

  # Setter for the current Client
  #
  # Returns the new Forawrd::Client instance
  def self.tunnel=(tunnel)
    @tunnel = tunnel
  end

  # Getter for the current Client
  #
  # Returns a Forward::Client instance
  def self.tunnel
    @tunnel
  end

  # Helper method for making forward quiet (silence most output)
  def self.quiet!
    @quiet = true
  end

  # Helper method for making forward quiet (silence most output)
  def self.quiet?
    @quiet == true
  end

  # Helper method for setting the log level to debug
  def self.debug!
    logger.level = Logger::DEBUG
  end

  # Setter for the logger
  #
  # Returns the new Logger instance
  def self.logger=(logger)
    @logger ||= logger
  end

  # Getter for the logger
  #
  # Returns the new or cached Logger instance
  def self.logger
    @logger ||= begin
      _logger           = Logger.new(STDOUT)
      _logger.level     = Logger::WARN
      _logger.formatter = proc do |severity, datetime, progname, msg|
        "#{severity} - [#{datetime.strftime('%H:%M:%S')}] #{msg}\n"
      end

      _logger
    end
  end

  # Returns a string representing a detailed client version
  #
  # Returns a String representing the client
  def self.client_string
    os     = RbConfig::CONFIG['host_os']
    engine = defined?(RUBY_ENGINE) ? RUBY_ENGINE : 'ruby'

    # TODO: make os version more friendly
    "(#{engine}-#{RUBY_VERSION})(#{os})(v#{Forward::VERSION})"
  end
end
