require 'cgi'
require 'json'
require 'net/http'
require 'net/https'
require 'uri'

module Forward
  module API
    class Resource
      include Common
      extend Common

      DEFAULT_ERROR_MESSAGE = "Unable to connect to API, please contact support@forwardhq.com".freeze

      attr_accessor :http
      attr_accessor :uri

      class JSONify
        def response(resp)
            resp.response = JSON.parse(resp.response, symbolize_names: true)
          rescue JSON::ParserError
            Forward.logger.debug 'Unable to parse API response'
        end
      end

      def initialize
        @http = EM::HttpRequest.new(Forward::API.host)

        @http.use JSONify
      end

      def request(method = :get, options = {}, &block)
        logger.debug "[API] request: #{method.to_s.upcase} #{@http.uri}#{options[:path]}"
        @options = options || {}

        add_head!
        add_body!

        @_request = @http.send(method, @options)

        @_request.callback {
          status = @_request.response_header.status

          logger.debug "[API] response: #{status} #{@_request.response}"
          if @_request.response.is_a?(String) && !@_request.response.empty?
            # no JSON response
            exit_with_error DEFAULT_ERROR_MESSAGE
          else
            block.call(@_request.response, status)
          end
        }

        @_request.errback {
          exit_with_error DEFAULT_ERROR_MESSAGE
        }
      end

      def get(options = {}, &block)
        request(:get, options, &block)
      end

      def post(options = {}, &block)
        request(:post, options, &block)
      end

      def delete(options = {}, &block)
        request(:delete, options, &block)
      end

      def add_body!
        return unless @options.has_key?(:params)

        logger.debug "[API] request params: #{@options[:params].inspect}"
        @options[:body] = @options.delete(:params).to_json
      end

      def add_head!
        @options[:head] ||= {
          'content-type' => 'application/json',
          'accept'       => 'application/json'
        }
        @options[:head]['authorization'] = "Token token=#{config.api_key}" if @options[:authenticated]
      end

    end

  end
end
