module Forward
  module Command
    class Base
      include Forward::Common

      attr_accessor :options
      attr_accessor :args

      def self.run(command, options = {}, args = [])
        Forward.logger.debug "[CLI] running `#{command}'"
        new(options, args).send(command)
      end

      def initialize(options = {}, args = [])
        @opts    = options
        @options = options.to_hash
        @args    = args
        logger.debug "[CLI] options: #{@options.inspect}"
        logger.debug "[CLI] args: #{@args.inspect}"
      end

      private

      def client(&block)
        EM.run {
          yield
          Signal.trap('INT')  { EM.stop; exit }
          Signal.trap('TERM') { EM.stop; exit }
        }
      end

      def ask_for_credentials(email = nil)
        if email.nil? || email !~ EMAIL_REGEX
          puts "Forward requires an account on #{HighLine.color('forwardhq.com', :underline)}"
          puts "Enter your email and password"
          email = ask('email: ').chomp
        end

        password = ask('password: ') { |q| q.echo = false }.chomp

        [email, password]
      end

      def print_help_and_exit!
        puts @opts
        exit
      end

      def validate(*validators)
        validators.each do |validator|
          validator = "validate_#{validator}"

          if respond_to?(validator, true)
            send(validator)
          else
            raise UnknownValidator, "Unable to find validator `#{validator}' in #{self.class.name}"
          end
        end
      end
    end
  end
end