module Forward
  module Command
    class Config < Base
      CONFIGURABLE_SETTINGS = %w[auto_copy auto_open].freeze
      TRUTHY_REGEX          = /\A(?:true|t|yes|y|1)\z/i.freeze
      FALESY_REGEX          = /\A(?:false|f|no|n|0)\z/i.freeze

      def set(*args)
        @setting = args[0]
        @value   = args[1]

        validate :setting, :value
        config.load
        config.send("#{@setting}=", @value)
        config.write
        exit_with_message "#{@setting} is now #{config.send(@setting)}"
      end

      def unset(*args)
        @setting = args.first

        validate :setting
        config.load
        default_value = config.set_default!(@setting)
        config.write
        exit_with_message "#{@setting} is now set to the default `#{default_value}'"
      end

      def get(*args)
        @setting = args.first

        validate :setting
        config.load
        exit_with_message "#{@setting} is currently #{config.send(@setting)}"
      end

      private

      def validate_value
        @value = @value.dup.strip

        case @setting
        when 'auto_copy'
          booleanize_value
        when 'auto_open'
          booleanize_value
        end
      end

      def booleanize_value
        return @value = true if @value =~ TRUTHY_REGEX
        return @value = false if @value =~ FALESY_REGEX

        raise ValidationError, "#{@setting} cannot be set to `#{@value}'"
      end

      def validate_setting
        unless CONFIGURABLE_SETTINGS.include?(@setting)
          raise ValidationError, "`#{@setting}' is an unknown setting"
        end
      end
    end
  end
end
