module Forward
  class Request
    include Common

    attr_accessor :tunnel
    attr_accessor :id
    attr_accessor :method
    attr_accessor :path
    attr_accessor :body
    attr_accessor :headers

    def initialize(tunnel, attributes)
      @tunnel      = tunnel
      @id          = attributes[:id]
      @method      = attributes[:method]
      @path        = attributes[:url]
      @headers     = attributes[:headers]
      @body        = ''
    end

    def url
      @url ||= "http://#{@tunnel.host}:#{@tunnel.port}"
    end

    def <<(data)
      @body << data

      @body
    end

    def destroy
      @tunnel.requests.delete(@id)
    end

    def process
      options = {
        path: path,
        body: (body.empty? ? nil : body),
        head: headers
      }

      logger.debug "[request] #{method} #{url}#{path} #{options.inspect}"
      puts "[#{Time.now.strftime('%H:%M:%S')}] [#{method}] #{path}" unless Forward.quiet?

      http = EM::HttpRequest.new(url, :inactivity_timeout => 60).send(method.downcase, options)

      http.headers { |header|
        # It's no longer gzipped, clear it
        header.raw.delete('Content-Encoding') if header.raw.has_key?('Content-Encoding')
        tunnel.socket.send(type: 'response:start', data: { id: id, headers: header.raw, status: header.status })
      }

      http.stream { |chunk|
        tunnel.socket.send({type: 'response:data', data: { id: id }}, chunk)
      }

      http.callback {
        tunnel.socket.send(type: 'response:end', data: { id: id })
        destroy
      }

      http.errback {
        handle_error(http.error)
        destroy
      }
    end

    def handle_error(error)
      puts "\e[31mhttp://#{tunnel.authority} isn't responding, make sure your server is running on localhost.\e[0m"
      tunnel.socket.send(type: 'response:error', data: { id: id, error_type: 'localhost_unavailable' })

      destroy
    end
  end
end
